@@ -32,7 +32,7 @@ sub list {
             $paths->row( $display_path, "/$action" );
         }
     }
-    $c->log->debug( "Loaded Path actions:\n" . $paths->draw )
+    $c->log->debug( "Loaded Path actions:\n" . $paths->draw . "\n" )
       if ( keys %{ $self->{paths} } );
 }
 
