@@ -1,9 +1,11 @@
 package Catalyst::Engine::CGI;
 
-use Moose;
-extends 'Catalyst::Engine';
+use strict;
+use base 'Catalyst::Engine';
+use NEXT;
+use URI;
 
-has _header_buf => (is => 'rw', clearer => '_clear_header_buf', predicate => '_has_header_buf');
+__PACKAGE__->mk_accessors('env');
 
 =head1 NAME
 
@@ -41,7 +43,8 @@ sub finalize_headers {
 
     $c->response->header( Status => $c->response->status );
 
-    $self->_header_buf($c->response->headers->as_string("\015\012") . "\015\012");
+    print $c->response->headers->as_string("\015\012") if *STDOUT->opened();
+    print "\015\012" if *STDOUT->opened();
 }
 
 =head2 $self->prepare_connection($c)
@@ -52,40 +55,34 @@ sub prepare_connection {
     my ( $self, $c ) = @_;
     local (*ENV) = $self->env || \%ENV;
 
-    my $request = $c->request;
-    $request->address( $ENV{REMOTE_ADDR} );
+    $c->request->address( $ENV{REMOTE_ADDR} );
 
   PROXY_CHECK:
     {
-        unless ( ref($c)->config->{using_frontend_proxy} ) {
+        unless ( $c->config->{using_frontend_proxy} ) {
             last PROXY_CHECK if $ENV{REMOTE_ADDR} ne '127.0.0.1';
-            last PROXY_CHECK if ref($c)->config->{ignore_frontend_proxy};
+            last PROXY_CHECK if $c->config->{ignore_frontend_proxy};
         }
         last PROXY_CHECK unless $ENV{HTTP_X_FORWARDED_FOR};
 
         # If we are running as a backend server, the user will always appear
         # as 127.0.0.1. Select the most recent upstream IP (last in the list)
         my ($ip) = $ENV{HTTP_X_FORWARDED_FOR} =~ /([^,\s]+)$/;
-        $request->address($ip);
-        if ( defined $ENV{HTTP_X_FORWARDED_PORT} ) {
-            $ENV{SERVER_PORT} = $ENV{HTTP_X_FORWARDED_PORT};
-        }
+        $c->request->address($ip);
     }
 
-    $request->hostname( $ENV{REMOTE_HOST} ) if exists $ENV{REMOTE_HOST};
-    $request->protocol( $ENV{SERVER_PROTOCOL} );
-    $request->user( $ENV{REMOTE_USER} );  # XXX: Deprecated. See Catalyst::Request for removal information
-    $request->remote_user( $ENV{REMOTE_USER} );
-    $request->method( $ENV{REQUEST_METHOD} );
+    $c->request->hostname( $ENV{REMOTE_HOST} );
+    $c->request->protocol( $ENV{SERVER_PROTOCOL} );
+    $c->request->user( $ENV{REMOTE_USER} );
+    $c->request->method( $ENV{REQUEST_METHOD} );
 
     if ( $ENV{HTTPS} && uc( $ENV{HTTPS} ) eq 'ON' ) {
-        $request->secure(1);
+        $c->request->secure(1);
     }
 
     if ( $ENV{SERVER_PORT} == 443 ) {
-        $request->secure(1);
+        $c->request->secure(1);
     }
-    binmode(STDOUT); # Ensure we are sending bytes.
 }
 
 =head2 $self->prepare_headers($c)
@@ -95,12 +92,12 @@ sub prepare_connection {
 sub prepare_headers {
     my ( $self, $c ) = @_;
     local (*ENV) = $self->env || \%ENV;
-    my $headers = $c->request->headers;
+
     # Read headers from %ENV
     foreach my $header ( keys %ENV ) {
         next unless $header =~ /^(?:HTTP|CONTENT|COOKIE)/i;
         ( my $field = $header ) =~ s/^HTTPS?_//;
-        $headers->header( $field => $ENV{$header} );
+        $c->req->headers->header( $field => $ENV{$header} );
     }
 }
 
@@ -127,9 +124,9 @@ sub prepare_path {
     # If we are running as a backend proxy, get the true hostname
   PROXY_CHECK:
     {
-        unless ( ref($c)->config->{using_frontend_proxy} ) {
+        unless ( $c->config->{using_frontend_proxy} ) {
             last PROXY_CHECK if $host !~ /localhost|127.0.0.1/;
-            last PROXY_CHECK if ref($c)->config->{ignore_frontend_proxy};
+            last PROXY_CHECK if $c->config->{ignore_frontend_proxy};
         }
         last PROXY_CHECK unless $ENV{HTTP_X_FORWARDED_HOST};
 
@@ -138,56 +135,42 @@ sub prepare_path {
         # backend could be on any port, so
         # assume frontend is on the default port
         $port = $c->request->secure ? 443 : 80;
-        if ( $ENV{HTTP_X_FORWARDED_PORT} ) {
-            $port = $ENV{HTTP_X_FORWARDED_PORT};
-        }
     }
 
-    # set the request URI
     my $path = $base_path . ( $ENV{PATH_INFO} || '' );
     $path =~ s{^/+}{};
 
-    # Using URI directly is way too slow, so we construct the URLs manually
-    my $uri_class = "URI::$scheme";
+    my $uri = URI->new;
+    $uri->scheme($scheme);
+    $uri->host($host);
+    $uri->port($port);
+    $uri->path($path);
+    $uri->query( $ENV{QUERY_STRING} ) if $ENV{QUERY_STRING};
 
-    # HTTP_HOST will include the port even if it's 80/443
-    $host =~ s/:(?:80|443)$//;
-
-    if ( $port !~ /^(?:80|443)$/ && $host !~ /:/ ) {
-        $host .= ":$port";
-    }
-
-    # Escape the path
-    $path =~ s/([^$URI::uric])/$URI::Escape::escapes{$1}/go;
-    $path =~ s/\?/%3F/g; # STUPID STUPID SPECIAL CASE
-
-    my $query = $ENV{QUERY_STRING} ? '?' . $ENV{QUERY_STRING} : '';
-    my $uri   = $scheme . '://' . $host . '/' . $path . $query;
-
-    $c->request->uri( bless \$uri, $uri_class );
+    # sanitize the URI
+    $uri = $uri->canonical;
+    $c->request->uri($uri);
 
     # set the base URI
     # base must end in a slash
-    $base_path .= '/' unless $base_path =~ m{/$};
-
-    my $base_uri = $scheme . '://' . $host . $base_path;
-
-    $c->request->base( bless \$base_uri, $uri_class );
+    $base_path .= '/' unless ( $base_path =~ /\/$/ );
+    my $base = $uri->clone;
+    $base->path_query($base_path);
+    $c->request->base($base);
 }
 
 =head2 $self->prepare_query_parameters($c)
 
 =cut
 
-around prepare_query_parameters => sub {
-    my $orig = shift;
+sub prepare_query_parameters {
     my ( $self, $c ) = @_;
     local (*ENV) = $self->env || \%ENV;
 
     if ( $ENV{QUERY_STRING} ) {
-        $self->$orig( $c, $ENV{QUERY_STRING} );
+        $self->SUPER::prepare_query_parameters( $c, $ENV{QUERY_STRING} );
     }
-};
+}
 
 =head2 $self->prepare_request($c, (env => \%env))
 
@@ -207,28 +190,14 @@ Enable autoflush on the output handle for CGI-based engines.
 
 =cut
 
-around prepare_write => sub {
+sub prepare_write {
+    my ( $self, $c ) = @_;
+
+    # Set the output handle to autoflush
     *STDOUT->autoflush(1);
-    return shift->(@_);
-};
 
-=head2 $self->write($c, $buffer)
-
-Writes the buffer to the client.
-
-=cut
-
-around write => sub {
-    my $orig = shift;
-    my ( $self, $c, $buffer ) = @_;
-
-    # Prepend the headers if they have not yet been sent
-    if ( $self->_has_header_buf ) {
-        $buffer = $self->_clear_header_buf . $buffer;
-    }
-
-    return $self->$orig( $c, $buffer );
-};
+    $self->NEXT::prepare_write($c);
+}
 
 =head2 $self->read_chunk($c, $buffer, $length)
 
@@ -240,22 +209,25 @@ sub read_chunk { shift; shift; *STDIN->sysread(@_); }
 
 =cut
 
-sub run { shift; shift->handle_request( env => \%ENV ) }
+sub run { shift; shift->handle_request(@_) }
 
 =head1 SEE ALSO
 
-L<Catalyst>, L<Catalyst::Engine>
+L<Catalyst> L<Catalyst::Engine>.
 
 =head1 AUTHORS
 
-Catalyst Contributors, see Catalyst.pm
+Sebastian Riedel, <sri@cpan.org>
+
+Christian Hansen, <ch@ngmedia.com>
+
+Andy Grundman, <andy@hybridized.org>
 
 =head1 COPYRIGHT
 
-This library is free software. You can redistribute it and/or modify it under
+This program is free software, you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
-no Moose;
 
 1;
