@@ -1,24 +1,31 @@
 package Catalyst::Engine::HTTP::Restarter::Watcher;
 
-use Moose;
-with 'MooseX::Emulate::Class::Accessor::Fast';
-
+use strict;
+use warnings;
+use base 'Class::Accessor::Fast';
 use File::Find;
 use File::Modified;
 use File::Spec;
 use Time::HiRes qw/sleep/;
 
-has delay => (is => 'rw');
-has regex => (is => 'rw');
-has modified => (is => 'rw');
-has directory => (is => 'rw');
-has watch_list => (is => 'rw');
-has follow_symlinks => (is => 'rw');
+__PACKAGE__->mk_accessors(
+    qw/delay
+      directory
+      modified
+      regex
+      watch_list/
+);
 
-no Moose;
+sub new {
+    my ( $class, %args ) = @_;
 
-sub BUILD {
-  shift->_init;
+    my $self = {%args};
+
+    bless $self, $class;
+
+    $self->_init;
+
+    return $self;
 }
 
 sub _init {
@@ -40,10 +47,8 @@ sub watch {
 
     my @changes;
     my @changed_files;
-    
-    my $delay = ( defined $self->delay ) ? $self->delay : 1;
 
-    sleep $delay if $delay > 0;
+    sleep $self->delay || 1;
 
     eval { @changes = $self->modified->changed };
     if ($@) {
@@ -95,9 +100,7 @@ sub watch {
 sub _index_directory {
     my $self = shift;
 
-    my $dir   = $self->directory;
-    die "No directory specified" if !$dir or ref($dir) && !@{$dir};
-
+    my $dir   = $self->directory || die "No directory specified";
     my $regex = $self->regex     || '\.pm$';
     my %list;
 
@@ -115,10 +118,9 @@ sub _index_directory {
                 $cur_dir =~ s{/script/..}{};
                 $list{$cur_dir} = 1;
             },
-            follow_fast => $self->follow_symlinks ? 1 : 0,
             no_chdir => 1
         },
-        ref $dir eq 'ARRAY' ? @{$dir} : $dir
+        $dir
     );
     return \%list;
 }
@@ -149,7 +151,7 @@ files
 
     my $watcher = Catalyst::Engine::HTTP::Restarter::Watcher->new(
         directory => '/path/to/MyApp',
-        regex     => '\.yml$|\.yaml$|\.conf|\.pm$',
+        regex     => '\.yml$|\.yaml$|\.pm$',
         delay     => 1,
     );
     
@@ -180,7 +182,9 @@ L<Catalyst>, L<Catalyst::Engine::HTTP::Restarter>, L<File::Modified>
 
 =head1 AUTHORS
 
-Catalyst Contributors, see Catalyst.pm
+Sebastian Riedel, <sri@cpan.org>
+
+Andy Grundman, <andy@hybridized.org>
 
 =head1 THANKS
 
