@@ -1,7 +1,7 @@
 package TestApp::Controller::Engine::Request::URI;
 
 use strict;
-use base 'Catalyst::Controller';
+use base 'Catalyst::Base';
 
 sub default : Private {
     my ( $self, $c ) = @_;
@@ -63,40 +63,4 @@ sub uri_with_utf8 : Local {
     $c->forward('TestApp::View::Dump::Request');
 }
 
-sub uri_with_undef : Local {
-    my ( $self, $c ) = @_;
-
-    my $warnings = 0;
-    local $SIG{__WARN__} = sub { $warnings++ };
-
-    # change the current uri
-    my $uri = $c->req->uri_with( { foo => undef } );
-    
-    $c->res->header( 'X-Catalyst-warnings' => $warnings );
-    
-    $c->forward('TestApp::View::Dump::Request');
-}
-
-sub uri_with_undef_only : Local {
-    my ( $self, $c ) = @_;
-
-    my $uri = $c->req->uri_with( { a => undef } );
-    
-    $c->res->header( 'X-Catalyst-uri-with' => "$uri" );
-    $c->forward('TestApp::View::Dump::Request');
-}
-
-sub uri_with_undef_ignore : Local {
-    my ( $self, $c ) = @_;
-
-    my $uri = $c->req->uri_with( { a => 1, b => undef } );
-    
-    my %query = $uri->query_form;
-    $c->res->header( 'X-Catalyst-uri-with' => "$uri" );
-    $c->res->header( 'X-Catalyst-Param-a' => $query{ a } );
-    $c->res->header( 'X-Catalyst-Param-b' => $query{ b } );
-    $c->res->header( 'X-Catalyst-Param-c' => $query{ c } );
-    $c->forward('TestApp::View::Dump::Request');
-}
-
 1;
