@@ -5,15 +5,15 @@ use base 'Catalyst::Base';
 
 sub one : Local {
     my ( $self, $c ) = @_;
-    $c->res->cookies->{catalyst} = { value => 'cool',     path => '/bah' };
-    $c->res->cookies->{cool}     = { value => 'catalyst', path => '/' };
+    $c->res->cookies->{Catalyst} = { value => 'Cool',     path => '/bah' };
+    $c->res->cookies->{Cool}     = { value => 'Catalyst', path => '/' };
     $c->forward('TestApp::View::Dump::Request');
 }
 
 sub two : Local {
     my ( $self, $c ) = @_;
-    $c->res->cookies->{catalyst} = { value => 'cool',     path => '/bah' };
-    $c->res->cookies->{cool}     = { value => 'catalyst', path => '/' };
+    $c->res->cookies->{Catalyst} = { value => 'Cool',     path => '/bah' };
+    $c->res->cookies->{Cool}     = { value => 'Catalyst', path => '/' };
     $c->res->redirect('http://www.google.com/');
 }
 
